//
// C++ Implementation: findfiles
//
// Description:
//
//
// Author: minsu kim <minsu.kim@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "global.h"

#include "findfilesdlg.h"

#include "kalzipapp.h"

#include <QtGui>

FindFilesDlg::FindFilesDlg(KalzipApp *app, QWidget *parent)
 : QDialog(parent)
 , m_app(app)
{
  setupUi(this);

  initLayout();
  initConnect();
}

FindFilesDlg::~FindFilesDlg()
{
}

void FindFilesDlg::initLayout()
{
  //QStringList labels;
  //labels << tr("File Name") << tr("Size");
  //filesTable->setHorizontalHeaderLabels(labels);
  TW_FindResult->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
  TW_FindResult->verticalHeader()->hide();
  TW_FindResult->setShowGrid(false);
}

void FindFilesDlg::initConnect()
{
  connect(PB_Browse, SIGNAL(clicked()), this, SLOT(browse()));
}

void FindFilesDlg::browse()
{
    QString directory = QFileDialog::getExistingDirectory(this,
                               tr("Find Files"), QDir::currentPath());
    if (!directory.isEmpty()) {
        CB_InDirectory->addItem(directory);
        //CB_InDirectory->setCurrentIndex(CB_InDirectory->currentIndex() + 1);
        CB_InDirectory->setCurrentIndex(CB_InDirectory->count()-1);
    }
}

void FindFilesDlg::find()
{
  TW_FindResult->setRowCount(0);

  QString fileName = CB_Named->currentText();
  QString text = CB_ContainingText->currentText();
  QString path = CB_InDirectory->currentText();

  QDir directory = QDir(path);
  QStringList files;
  if (fileName.isEmpty())
      fileName = "*";
  files = directory.entryList(QStringList(fileName),
                              QDir::Files | QDir::NoSymLinks);

  if (!text.isEmpty())
      files = findFiles(directory, files, text);
  showFiles(directory, files);
}

QStringList FindFilesDlg::findFiles(const QDir &directory, const QStringList &files, const QString &text)
{
    QProgressDialog progressDialog(this);
    progressDialog.setCancelButtonText(tr("&Cancel"));
    progressDialog.setRange(0, files.size());
    progressDialog.setWindowTitle(tr("Find Files"));

    QStringList foundFiles;

    for (int i = 0; i < files.size(); ++i) {
        progressDialog.setValue(i);
        progressDialog.setLabelText(tr("Searching file number %1 of %2...")
                                    .arg(i).arg(files.size()));
        qApp->processEvents();

        if (progressDialog.wasCanceled())
            break;

        QFile file(directory.absoluteFilePath(files[i]));

        if (file.open(QIODevice::ReadOnly)) {
            QString line;
            QTextStream in(&file);
            while (!in.atEnd()) {
                if (progressDialog.wasCanceled())
                    break;
                line = in.readLine();
                if (line.contains(text)) {
                    foundFiles << files[i];
                    break;
                }
            }
        }
    }
    return foundFiles;
}

void FindFilesDlg::showFiles(const QDir &directory, const QStringList &files)
{
    for (int i = 0; i < files.size(); ++i) {
        QFile file(directory.absoluteFilePath(files[i]));
        qint64 size = QFileInfo(file).size();

        QTableWidgetItem *fileNameItem = new QTableWidgetItem(files[i]);
        fileNameItem->setFlags(Qt::ItemIsEnabled);
        QTableWidgetItem *sizeItem = new QTableWidgetItem(tr("%1 KB")
                                             .arg(int((size + 1023) / 1024)));
        sizeItem->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        sizeItem->setFlags(Qt::ItemIsEnabled);

        int row = TW_FindResult->rowCount();
        TW_FindResult->insertRow(row);
        TW_FindResult->setItem(row, 0, fileNameItem);
        TW_FindResult->setItem(row, 1, sizeItem);
    }
    //filesFoundLabel->setText(tr("%1 file(s) found").arg(files.size()));
}

void FindFilesDlg::on_PB_Search_clicked()
{
  find();
  //accept();
}

void FindFilesDlg::on_PB_Cancel_clicked()
{
  close();
  //reject();
}

void FindFilesDlg::on_TW_FindResult_itemDoubleClicked(QTableWidgetItem * item)
{

}

void FindFilesDlg::on_TW_FindResult_cellDoubleClicked(int row, int column)
{
  QFileInfo fi(TW_FindResult->item(row, 0)->text()); // get filename path

  if ( fi.exists() and fi.isFile() ) {
    if ( fi.suffix() == "alz" or fi.suffix() == "ALZ" ) {
      emit sigExtract(fi.absoluteFilePath());
      close();
    } else { // not ALZip file
      QMessageBox::warning(this, "Warning", "This is not ALZip file.\nPlease select to the other file.");
    }
  }
}

void FindFilesDlg::showEvent(QShowEvent * event)
{
  // is not visible?
  activateWindow();
  return QDialog::showEvent(event);
}
