//
// C++ Implementation: mainwindow
//
// Description:
//
//
// Author: minsu kim <minsu.kim@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "global.h"

#include "kalzipapp.h"

#include "mainwindow.h"
#include "wizarddlg.h"
#include "informationdlg.h"
#include "aboutdlg.h"
#include "findfilesdlg.h"

#include <stdio.h>
#include <iostream>
#include <time.h>
#include <string>
#include <vector>
#include <locale.h>
#include <sys/timeb.h>

#include "UnAlz.h"
#include "UnAlzUtils.h"

#include <QtGui>

extern CUnAlz g_unalzCore;

MainWindow::MainWindow(KalzipApp *app, QWidget *parent)
 : QMainWindow(parent)
 , m_app(app)
 , m_FI_Tree(0)
{
  setupUi(this);
  
  initLayout();
  initTrayIcon();
}

MainWindow::~MainWindow()
{
}

void MainWindow::initLayout()
{
  createToolBar();
  createFileManager();  
}

void MainWindow::on_TV_DirView_clicked(const QModelIndex & idx)
{
  if ( m_FI_Tree ) {
    delete m_FI_Tree;
    m_FI_Tree = 0;
  }

  m_FI_Tree = new QFileInfo(m_DM_Tree->fileInfo(idx));
  qDebug() << m_FI_Tree->absoluteFilePath();
}

void MainWindow::on_TV_DirView_doubleClicked(const QModelIndex & index)
{

}

void MainWindow::slotChangeUpDirectory()
{
  //TV_DirView->setRootIndex(m_DM_Tree->index("/home/minsu"));
}

///////////////////////////////////
// ToolBar IconMenu slot functions
///////////////////////////////////

void MainWindow::slotOpenArchive()
{
  on_AC_Open_archive_triggered(TRUE);
}

void MainWindow::slotExtractTo()
{
  on_AC_Extract_to_triggered(TRUE); 
}

void MainWindow::slotTest()
{
}

void MainWindow::slotView()
{
}

void MainWindow::slotDelete()
{
}

void MainWindow::slotFind()
{
  m_app->getFindFilesDlg()->show();
}

void MainWindow::slotInfo()
{
  //m_app->getInformationDlg()->show();
}

void MainWindow::slotWizard()
{
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (m_trayIcon->isVisible()) {
        QMessageBox::information(this, tr("Systray"),
                                  tr("The program will keep running in the "
                                    "system tray. To terminate the program, "
                                    "choose <b>Quit</b> in the context menu "
                                    "of the system tray entry."));
        hide();
        event->ignore();
    }
}

void MainWindow::initTrayIcon()
{
  QAction *minimizeAction = new QAction(tr("Mi&nimize"), this);
  connect(minimizeAction, SIGNAL(triggered()), this, SLOT(hide()));

  QAction *maximizeAction = new QAction(tr("Ma&ximize"), this);
  connect(maximizeAction, SIGNAL(triggered()), this, SLOT(showMaximized()));

  QAction *restoreAction = new QAction(tr("&Restore"), this);
  connect(restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));

  QAction *quitAction = new QAction(tr("&Quit"), this);
  connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

  QMenu *trayIconMenu = new QMenu(this);
  trayIconMenu->addAction(minimizeAction);
  trayIconMenu->addAction(maximizeAction);
  trayIconMenu->addAction(restoreAction);
  trayIconMenu->addSeparator();
  trayIconMenu->addAction(quitAction);

  QIcon icon(QPixmap("images/kalzip_logo.png"));
  setWindowIcon(icon);

  m_trayIcon = new QSystemTrayIcon(this);
  m_trayIcon->setIcon(icon);
  m_trayIcon->setContextMenu(trayIconMenu);

  m_trayIcon->show();
  connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
          this, SLOT(slotTrayActivated(QSystemTrayIcon::ActivationReason)));

  m_trayIcon->setToolTip("KALZip");
}

void MainWindow::slotTrayActivated(QSystemTrayIcon::ActivationReason reason)
{
  switch (reason) {
    case QSystemTrayIcon::Trigger: {
      if ( isVisible() == TRUE )
        hide();
      else
        show();
    } break;
    case QSystemTrayIcon::DoubleClick: break;
    case QSystemTrayIcon::MiddleClick: break;
    default:
        ;
    }
}

/////////////
// File Menu
/////////////

void MainWindow::on_AC_Open_archive_triggered(bool checked)
{
#if 1
  QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), QDir::currentPath(), " ALZip (*.alz)");

  g_unalzCore.SetPipeMode(FALSE);

  g_unalzCore.Open(fileName.toLatin1().data());
  ListAlzForGui(&g_unalzCore, fileName.toLatin1().data());
#endif                                                            
}

void MainWindow::on_AC_Exit_triggered(bool checked)
{
  m_app->quit();
}

/////////////////
// Commands Menu
/////////////////

void MainWindow::on_AC_Extract_to_triggered(bool checked)
{
#if 1
  QString destdir = QFileDialog::getExistingDirectory(this, tr("Open Directory"), QDir::currentPath(), QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);

  g_unalzCore.ExtractAll( destdir.toAscii().data() );

  QMessageBox::information(this, tr("Extract to.."), destdir + tr(" Completed.") );
#endif                                                                
}

//////////////
// Tools Menu
//////////////

void MainWindow::on_AC_Find_Files_triggered(bool checked)
{
  m_app->getFindFilesDlg()->show(); 
}

/////////////
// Help menu 
/////////////

void MainWindow::on_AC_About_Qt_triggered(bool checked)
{
  QApplication::aboutQt ();
}

void MainWindow::on_AC_About_triggered(bool checked)
{
  // version 0.10 unalz command process version.
  // version 0.20 first release intergrated Qt4.
  // version 0.30 release for oss 2008 contest.
  m_app->getAboutDlg()->show();
}

void MainWindow::createToolBar()
{
  QAction *a = new QAction( QIcon(QPixmap(":images/db_add.png")), "Open archive", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotOpenArchive()));

  a = new QAction( QIcon(QPixmap(":images/db_update.png")), "Extract To", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotExtractTo()));

  a = new QAction( QIcon(QPixmap(":images/db_status.png")), "Test", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotTest()));

  a = new QAction( QIcon(QPixmap(":images/db.png")), "View", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotView()));

  a = new QAction( QIcon(QPixmap(":images/trash.png")), "Delete", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotDelete()));

  a = new QAction( QIcon(QPixmap(":images/find.png")), "Find", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotFind()));

  a = new QAction( QIcon(QPixmap(":images/wizard.png")), "Wizard", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotWizard()));
  
  a = new QAction( QIcon(QPixmap(":images/info.png")), "Info", this );
  TB_IconMenu->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotInfo()));
  
  a = new QAction(QIcon(QPixmap(":images/up.png")), "Up", this);
  TB_CurDir->addAction(a);
  connect(a, SIGNAL(triggered()), this, SLOT(slotChangeUpDirectory()));

  TB_CurDir->addSeparator();
  m_CB_CurDir = new QComboBox();
  m_CB_CurDir->setEditable ( TRUE );

  TB_CurDir->addWidget(m_CB_CurDir);
  m_CB_CurDir->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
}

void MainWindow::createFileManager()
{
  m_DM_Tree = new QDirModel;

  TV_DirView->setModel(m_DM_Tree);
  TV_DirView->setRootIndex(m_DM_Tree->index(QDir::currentPath()));

  TV_DirView->resizeColumnToContents(0);
  TV_DirView->resizeColumnToContents(1);
  TV_DirView->resizeColumnToContents(2);
  TV_DirView->resizeColumnToContents(3);
}

#if 1
void MainWindow::ListAlzForGui(CUnAlz* pUnAlz, const char* src)
{
  CUnAlz::FileList::iterator  i;
  CUnAlz::FileList*     list;

  list = pUnAlz->GetFileList();

  //printf("\nListing archive: %s\n"
  //     "\n   Date & Time      Attr          Size   Compressed  Name\n",
  //     src);
  //printf("------------------- ----- ------------ ------------  ------------\n");

//  char szDate[64];
  char szTime[64];
  char szAttr[LEN_ATTR];
  UINT64 totalUnCompressedSize = 0;
  UINT64 totalCompressedSize = 0;
  unsigned fileNum = 0;
  time_t  time;
  struct tm*    filetm;

  QList<QTreeWidgetItem *> items;

  for(i=list->begin(); i<list->end(); i++)
  {
    // time
    time = dosTime2TimeT(i->head.fileTimeDate);
    filetm = localtime(&time);
//    strftime(szTime, 64, "%H:%M:%S", filetm);
//    strftime(szDate, 64, "%Y-%m-%d", filetm);
    strftime(szTime, 64, "%x %X", filetm);        // use system locale
    // attributes
    FileAttr2Str(szAttr, i->head.fileAttribute);

    QStringList filelst;

    //printf("%19s %s " I64FORM(12) " " I64FORM(12) "   %s%s\n",
    filelst << QString(i->fileName) << QDateTime::fromTime_t(time).toString(Qt::ISODate) << QString(szAttr)
    << QString::number(i->uncompressedSize) << QString::number(i->compressedSize);
    //<< i->head.fileDescriptor & ALZ_FILE_DESCRIPTOR_ENCRYPTED ? "*" : "";
    qTrace << filelst;
        
    //items.append(new QTreeWidgetItem((QTreeWidget*)0, filelst));
    TW_CurDir->addTopLevelItem(new QTreeWidgetItem((QTreeWidget*)0, filelst));
    qApp->processEvents();
    
    ++fileNum;
    totalUnCompressedSize += i->uncompressedSize;
    totalCompressedSize += i->compressedSize;
  }

  //TW_CurDir->insertTopLevelItems(0, items);
  //printf("------------------- ----- ------------ ------------  ------------\n");
  QString resultTotal;
  resultTotal.sprintf("Total Size: " U64FORM(12) " Compressed Size: " U64FORM(12) "   %u file(s)\n",
  totalUnCompressedSize, totalCompressedSize, fileNum);

  QStatusBar *sBar = statusBar ();
  sBar->showMessage( resultTotal, 0 );
  
  TW_CurDir->resizeColumnToContents(0);
  TW_CurDir->resizeColumnToContents(1);
  TW_CurDir->resizeColumnToContents(2);
  TW_CurDir->resizeColumnToContents(3);
  TW_CurDir->resizeColumnToContents(4);
}

#endif
                             